@@ -277,7 +277,9 @@ class TestObjMakeList(test.TestCase):
     def test_obj_make_list(self):
         @base.VersionedObjectRegistry.register
         class MyList(base.ObjectListBase, base.VersionedObject):
-            pass
+            fields = {
+                'objects': fields.ListOfObjectsField('MyObj'),
+            }
 
         db_objs = [{'foo': 1, 'bar': 'baz', 'missing': 'banana'},
                    {'foo': 2, 'bar': 'bat', 'missing': 'apple'},
@@ -459,13 +461,12 @@ class TestFixture(_BaseTestCase):
                          actual['TestSubclassedObject'])
 
     def test_test_compatibility(self):
-        registry = base.VersionedObjectRegistry()
-        checker = fixture.ObjectVersionChecker()
         fake_classes = {mock.sentinel.class_one: [mock.sentinel.impl_one_one,
                                                   mock.sentinel.impl_one_two],
                         mock.sentinel.class_two: [mock.sentinel.impl_two_one,
                                                   mock.sentinel.impl_two_two],
                         }
+        checker = fixture.ObjectVersionChecker(fake_classes)
 
         @mock.patch.object(checker, '_test_object_compatibility')
         def test(mock_compat):
@@ -476,9 +477,7 @@ class TestFixture(_BaseTestCase):
                  mock.call(mock.sentinel.impl_two_one),
                  mock.call(mock.sentinel.impl_two_two)],
                 any_order=True)
-
-        with mock.patch.object(registry, '_obj_classes', new=fake_classes):
-            test()
+        test()
 
     def test_test_compatibility_checks_obj_to_primitive(self):
         fake = mock.MagicMock()
@@ -493,13 +492,12 @@ class TestFixture(_BaseTestCase):
              mock.call(target_version='1.3')])
 
     def test_test_relationships_in_order(self):
-        registry = base.VersionedObjectRegistry()
-        checker = fixture.ObjectVersionChecker()
         fake_classes = {mock.sentinel.class_one: [mock.sentinel.impl_one_one,
                                                   mock.sentinel.impl_one_two],
                         mock.sentinel.class_two: [mock.sentinel.impl_two_one,
                                                   mock.sentinel.impl_two_two],
                         }
+        checker = fixture.ObjectVersionChecker(fake_classes)
 
         @mock.patch.object(checker, '_test_relationships_in_order')
         def test(mock_compat):
@@ -510,9 +508,7 @@ class TestFixture(_BaseTestCase):
                  mock.call(mock.sentinel.impl_two_one),
                  mock.call(mock.sentinel.impl_two_two)],
                 any_order=True)
-
-        with mock.patch.object(registry, '_obj_classes', new=fake_classes):
-            test()
+        test()
 
     def test_test_relationships_in_order_good(self):
         fake = mock.MagicMock()
@@ -1047,6 +1043,182 @@ class _TestObject(object):
         self.assertTrue(obj1 == obj2)
         self.assertFalse(obj1 == obj3)
 
+    def test_obj_list_fields_modifications(self):
+        @base.VersionedObjectRegistry.register
+        class ObjWithList(base.VersionedObject):
+            fields = {
+                'list_field': fields.Field(fields.List(fields.Integer)),
+            }
+        obj = ObjWithList()
+
+        def set_by_index(val):
+            obj.list_field[0] = val
+
+        def append(val):
+            obj.list_field.append(val)
+
+        def extend(val):
+            obj.list_field.extend([val])
+
+        def add(val):
+            obj.list_field = obj.list_field + [val]
+
+        def iadd(val):
+            """Test += corner case
+
+            a=a+b and a+=b use different magic methods under the hood:
+            first one calls __add__ which clones initial value before the
+            assignment, second one call __iadd__ which modifies the initial
+            list.
+            Assignment should cause coercing in both cases, but __iadd__ may
+            corrupt the initial value even if the assignment fails.
+            So it should be overridden as well, and this test is needed to
+            verify it
+            """
+            obj.list_field += [val]
+
+        def insert(val):
+            obj.list_field.insert(0, val)
+
+        def simple_slice(val):
+            obj.list_field[:] = [val]
+
+        def extended_slice(val):
+            """Extended slice case
+
+            Extended slice (and regular slices in py3) are handled differently
+            thus needing a separate test
+            """
+            obj.list_field[::2] = [val]
+
+        # positive tests to ensure that coercing works properly
+        obj.list_field = ["42"]
+        set_by_index("1")
+        append("2")
+        extend("3")
+        add("4")
+        iadd("5")
+        insert("0")
+        self.assertEqual([0, 1, 2, 3, 4, 5], obj.list_field)
+        simple_slice("10")
+        self.assertEqual([10], obj.list_field)
+        extended_slice("42")
+        self.assertEqual([42], obj.list_field)
+        obj.obj_reset_changes()
+        # negative tests with non-coerceable values
+        self.assertRaises(ValueError, set_by_index, "abc")
+        self.assertRaises(ValueError, append, "abc")
+        self.assertRaises(ValueError, extend, "abc")
+        self.assertRaises(ValueError, add, "abc")
+        self.assertRaises(ValueError, iadd, "abc")
+        self.assertRaises(ValueError, insert, "abc")
+        self.assertRaises(ValueError, simple_slice, "abc")
+        self.assertRaises(ValueError, extended_slice, "abc")
+        # ensure that nothing has been changed
+        self.assertEqual([42], obj.list_field)
+        self.assertEqual({}, obj.obj_get_changes())
+
+    def test_obj_dict_field_modifications(self):
+        @base.VersionedObjectRegistry.register
+        class ObjWithDict(base.VersionedObject):
+            fields = {
+                'dict_field': fields.Field(fields.Dict(fields.Integer)),
+            }
+        obj = ObjWithDict()
+        obj.dict_field = {"1": 1, "3": 3, "4": 4}
+
+        def set_by_key(key, value):
+            obj.dict_field[key] = value
+
+        def add_by_key(key, value):
+            obj.dict_field[key] = value
+
+        def update_w_dict(key, value):
+            obj.dict_field.update({key: value})
+
+        def update_w_kwargs(key, value):
+            obj.dict_field.update(**{key: value})
+
+        def setdefault(key, value):
+            obj.dict_field.setdefault(key, value)
+
+        # positive tests to ensure that coercing works properly
+        set_by_key("1", "10")
+        add_by_key("2", "20")
+        update_w_dict("3", "30")
+        update_w_kwargs("4", "40")
+        setdefault("5", "50")
+        self.assertEqual({"1": 10, "2": 20, "3": 30, "4": 40, "5": 50},
+                         obj.dict_field)
+        obj.obj_reset_changes()
+        # negative tests with non-coerceable values
+        self.assertRaises(ValueError, set_by_key, "key", "abc")
+        self.assertRaises(ValueError, add_by_key, "other", "abc")
+        self.assertRaises(ValueError, update_w_dict, "key", "abc")
+        self.assertRaises(ValueError, update_w_kwargs, "key", "abc")
+        self.assertRaises(ValueError, setdefault, "other", "abc")
+        # ensure that nothing has been changed
+        self.assertEqual({"1": 10, "2": 20, "3": 30, "4": 40, "5": 50},
+                         obj.dict_field)
+        self.assertEqual({}, obj.obj_get_changes())
+
+    def test_obj_set_field_modifications(self):
+        @base.VersionedObjectRegistry.register
+        class ObjWithSet(base.VersionedObject):
+            fields = {
+                'set_field': fields.Field(fields.Set(fields.Integer))
+            }
+        obj = ObjWithSet()
+        obj.set_field = set([42])
+
+        def add(value):
+            obj.set_field.add(value)
+
+        def update_w_set(value):
+            obj.set_field.update(set([value]))
+
+        def update_w_list(value):
+            obj.set_field.update([value, value, value])
+
+        def sym_diff_upd(value):
+            obj.set_field.symmetric_difference_update(set([value]))
+
+        def union(value):
+            obj.set_field = obj.set_field | set([value])
+
+        def iunion(value):
+            obj.set_field |= set([value])
+
+        def xor(value):
+            obj.set_field = obj.set_field ^ set([value])
+
+        def ixor(value):
+            obj.set_field ^= set([value])
+        # positive tests to ensure that coercing works properly
+        sym_diff_upd("42")
+        add("1")
+        update_w_list("2")
+        update_w_set("3")
+        union("4")
+        iunion("5")
+        xor("6")
+        ixor("7")
+        self.assertEqual(set([1, 2, 3, 4, 5, 6, 7]), obj.set_field)
+        obj.set_field = set([42])
+        obj.obj_reset_changes()
+        # negative tests with non-coerceable values
+        self.assertRaises(ValueError, add, "abc")
+        self.assertRaises(ValueError, update_w_list, "abc")
+        self.assertRaises(ValueError, update_w_set, "abc")
+        self.assertRaises(ValueError, sym_diff_upd, "abc")
+        self.assertRaises(ValueError, union, "abc")
+        self.assertRaises(ValueError, iunion, "abc")
+        self.assertRaises(ValueError, xor, "abc")
+        self.assertRaises(ValueError, ixor, "abc")
+        # ensure that nothing has been changed
+        self.assertEqual(set([42]), obj.set_field)
+        self.assertEqual({}, obj.obj_get_changes())
+
 
 class TestObject(_LocalTest, _TestObject):
     def test_set_defaults(self):
